"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Amounts_1;
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const pre_conditions_1 = require("../util/pre-conditions");
const amount_1 = require("./amount");
const jsonable_1 = require("./jsonable");
/**
 * Representation of all the amounts relevant for a transaction.
 *
 * See {@link Amount} for representation of a single amount value with associated currency.
 */
let Amounts = Amounts_1 = class Amounts extends jsonable_1.Jsonable {
    constructor() {
        super(...arguments);
        this.baseAmount = 0;
        this.additionalAmounts = {};
        this.currency = 'XXX';
        this.currencyExchangeRate = 0;
        this.originalCurrency = undefined;
    }
    /**
       * Convert a JSON string into an {@link Amounts} object if possible
       *
       * @param json The JSON to convert
       */
    static fromJson(json) {
        return jsonable_1.Jsonable.baseFromJson(json, Amounts_1);
    }
    /**
       * Initialise with base amount (inclusive of tax), currency and optional additional amounts map.
       *
       * @param baseAmount        The base amount, inclusive of tax, in subunit form (cents, pence, etc)
       * @param currency          The ISO-4217 currency code
       * @param additionalAmounts The additional amounts
       *
       * @returns The initialised amounts object
       */
    static from(baseAmount = 0, currency = 'XXX', additionalAmounts = {}) {
        const amounts = new Amounts_1();
        amounts.baseAmount = baseAmount;
        amounts.currency = currency;
        amounts.additionalAmounts = additionalAmounts;
        return amounts;
    }
    /**
       * Initialise a new Amounts object from another one.
       *
       * The method will create a new Amounts object cloned from the original
       *
       * @param from The Amounts object to clone
       *
       * @returns The initialised amounts object
       */
    static fromAmounts(from) {
        const obj = Amounts_1.from(from.baseAmount, from.currency, from.additionalAmounts);
        obj.currencyExchangeRate = from.currencyExchangeRate;
        obj.originalCurrency = from.originalCurrency;
        return obj;
    }
    /**
       * Add an additional amount to complement the base amount.
       *
       * The additional amounts are represented via a string identifier and the amount value in subunit form.
       *
       * Examples of identifiers are "tip" and "cashback". Note that identifiers are case sensitive!
       *
       * @param identifier The string identifier for the amount
       * @param amount     The amount value
       */
    addAdditionalAmount(identifier, amount) {
        pre_conditions_1.PreConditions.checkArgument(identifier != null && amount >= 0, 'Identifier must be set and value must be >= 0');
        this.additionalAmounts[identifier] = amount;
    }
    /**
       * Add an additional amount as a fraction of the base amount.
       *
       * This is useful for cases where a fee, charity contribution, etc is calculated as a fraction or percentage of the base amount value.
       *
       * Examples of identifiers are "tip" and "cashback". Note that identifiers are case sensitive!
       *
       * @param identifier The string identifier for the amount
       * @param fraction   The fraction of the base amount, ranging from 0.0 to 1.0f (0% to 100%)
       */
    addAdditionalAmountAsBaseFraction(identifier, fraction) {
        if (fraction < 0.0 || fraction > 1.0) {
            throw new pre_conditions_1.InvalidArgumentError('Fraction must be between 0.0 and 1.0');
        }
        this.addAdditionalAmount(identifier, this.baseAmount * fraction);
    }
    /**
       * Get an {@link Amount} representation of the base amount with associated currency.
       *
       * The base amount is inclusive of any tax.
       *
       * @return Base {@link Amount}
       */
    getBaseAmount() {
        return amount_1.Amount.from(this.baseAmount, this.currency);
    }
    /**
       * Check whether there is an additional amount with the provided identifier defined.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return True if one is defined, false otherwise
       */
    hasAdditionalAmount(identifier) {
        return this.additionalAmounts[identifier] !== undefined;
    }
    /**
       * Get the additional amount value for the provided identifier.
       *
       * If none is set, 0 will be returned.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return The amount value
       */
    getAdditionalAmountValue(identifier) {
        if (this.additionalAmounts[identifier]) {
            return this.additionalAmounts[identifier];
        }
        return 0;
    }
    /**
       * Get an {@link Amount} representation of the additional amount with associated currency.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return The additional {@link Amount}
       */
    getAdditionalAmount(identifier) {
        return amount_1.Amount.from(this.getAdditionalAmountValue(identifier), this.currency);
    }
    /**
       * Get the map of all the additional amounts set.
       *
       * Note that amount identifiers are case sensitive.
       *
       * @return The map of identifier keys mapped to amount values
       */
    getAdditionalAmounts() {
        return this.additionalAmounts;
    }
    /**
       * Get the total amount (base + additional amounts) in subunit form.
       *
       * @return The total amount
       */
    getTotalAmountValue() {
        let total = this.baseAmount;
        for (const key of Object.keys(this.additionalAmounts)) {
            total += this.additionalAmounts[key];
        }
        return total;
    }
    /**
       * Get the total amount (base + additional amounts) in subunit form, excluding any additionals as defined by their provided identifiers.
       *
       * This is useful for scenarios where some additional amounts are supported natively in an environment (such as tip and cashback), but others
       * (like charity donations) are not and should be appended to the base amount for that environment.
       *
       * Note that identifiers are case sensitive.
       *
       * @param amountIdentifiers The identifiers of the amounts to exclude from the calculation
       * @return The total amount value, excluding the amounts as provider via the identifiers
       */
    getTotalExcludingAmounts(...amountIdentifiers) {
        let total = this.baseAmount;
        for (const key of Object.keys(this.additionalAmounts)) {
            if (!amountIdentifiers.includes(key)) {
                total += this.additionalAmounts[key];
            }
        }
        return total;
    }
    /**
       * Get an {@link Amount} representation of the total amount with associated currency.
       *
       * @return Total {@link Amount}
       */
    getTotalAmount() {
        return amount_1.Amount.from(this.getTotalAmountValue(), this.currency);
    }
    /**
       * Add together two amounts and get a new Amounts instance back with the result.
       *
       * @param a1 The first amount to add with
       * @param a2 The second amount to add with
       * @return The combined result
       */
    static addAmounts(a1, a2) {
        if (a1 == null || a2 == null || a1.currency !== a2.currency) {
            throw new pre_conditions_1.InvalidArgumentError('Invalid amounts or trying to combine different currencies');
        }
        const newBaseAmount = a1.baseAmount + a2.baseAmount;
        const newAdditionals = a1.additionalAmounts;
        const a2Additionals = a2.additionalAmounts;
        for (const a2Key of Object.keys(a2Additionals)) {
            if (Object.prototype.hasOwnProperty.call(newAdditionals, a2Key)) {
                newAdditionals[a2Key] += a2Additionals[a2Key];
            }
            else {
                newAdditionals[a2Key] = a2Additionals[a2Key];
            }
        }
        return Amounts_1.from(newBaseAmount, a1.currency, newAdditionals);
    }
    /**
       * Subtract one amounts from another.
       *
       * Note that the result Amounts will only contain additionalAmounts defined in a1, the one being subtracted from.
       * If an amount is set in a2 only, it will not be added to the resulting Amounts.
       *
       * @param a1                        The amounts to subtract a2 from
       * @param a2                        The amounts of which a1 will be reduced by
       * @param keepZeroAmountAdditionals Whether or not to keep additional amounts with a value of zero
       * @return The reduced amounts
       */
    static subtractAmounts(a1, a2, keepZeroAmountAdditionals = true) {
        if (a1 == null || a2 == null || a1.currency !== a2.currency) {
            throw new pre_conditions_1.InvalidArgumentError('Invalid amounts or trying to combine different currencies');
        }
        const remainingBase = Math.max(a1.baseAmount - a2.baseAmount, 0);
        const newAdditionals = a1.additionalAmounts;
        const a2Additionals = a2.additionalAmounts;
        for (const a2Key of Object.keys(a2Additionals)) {
            if (Object.prototype.hasOwnProperty.call(newAdditionals, a2Key)) {
                const additionalRemainder = Math.max(newAdditionals[a2Key] - a2Additionals[a2Key], 0);
                if (additionalRemainder > 0 || keepZeroAmountAdditionals) {
                    newAdditionals[a2Key] = additionalRemainder;
                }
                else {
                    delete newAdditionals[a2Key];
                }
            }
        }
        return Amounts_1.from(remainingBase, a1.currency, newAdditionals);
    }
};
__decorate([
    json2typescript_1.JsonProperty('baseAmount'),
    __metadata("design:type", Number)
], Amounts.prototype, "baseAmount", void 0);
__decorate([
    json2typescript_1.JsonProperty('additionalAmounts'),
    __metadata("design:type", Object)
], Amounts.prototype, "additionalAmounts", void 0);
__decorate([
    json2typescript_1.JsonProperty('currency', String, true),
    __metadata("design:type", String)
], Amounts.prototype, "currency", void 0);
__decorate([
    json2typescript_1.JsonProperty('currencyExchangeRate', Number, true),
    __metadata("design:type", Number)
], Amounts.prototype, "currencyExchangeRate", void 0);
__decorate([
    json2typescript_1.JsonProperty('originalCurrency', String, true),
    __metadata("design:type", String)
], Amounts.prototype, "originalCurrency", void 0);
Amounts = Amounts_1 = __decorate([
    json2typescript_1.JsonObject('Amounts')
], Amounts);
exports.Amounts = Amounts;
