import { BasketItem } from './basket-item';
import { BasketItemModifier } from './basket-item-modifier';
import { AdditionalData } from './additional-data';
/**
 * Builder to create {@link BasketItem} instances.
 */
export declare class BasketItemBuilder {
    private id;
    private quantity;
    private label;
    private category;
    private baseAmount;
    private amount;
    private measurement?;
    private modifiers?;
    private itemData?;
    /**
       * Initialise the builder with a default random id.
       */
    constructor();
    /**
       * Initialise the builder from the provided basket item.
       *
       * @param copyFrom The item to copy from
       */
    static from(copyFrom: BasketItem): BasketItemBuilder;
    /**
       * Generate a new random id (UUID) for this item.
       *
       * Note that the builder is initialised with a random id, meaning this only has to be called to generate a new random id.
       *
       * @return This builder
       */
    generateRandomId(): BasketItemBuilder;
    /**
       * Set the id for this item, overriding the default generated random id.
       *
       * @param id The id
       * @return This builder
       */
    withId(id: string): BasketItemBuilder;
    /**
       * Set the quantity for this item.
       *
       * Defaults to 1 if not set.
       *
       * See [[withMeasurement]] and [[withFractionalQuantity]] for scenarios where quantities have or may have a fractional part and a unit.
       *
       * @param quantity The item quantity
       * @return This builder
       */
    withQuantity(quantity: number): BasketItemBuilder;
    /**
       * Increment the quantity by one.
       *
       * @return This builder
       */
    incrementQuantity(): BasketItemBuilder;
    /**
       * Decrements the item quantity by one as long as the current quantity is greater than zero.
       *
       * If the quantity is already zero, this method has no effect.
       *
       * @return This builder
       */
    decrementQuantity(): BasketItemBuilder;
    /**
       * Modify the current quantity with the provided offset.
       *
       * This effectively does quantity += offset, and can be used to increase or decrease the quantity.
       *
       * @param offset The value to modify the current quantity with.
       * @return This builder
       */
    offsetQuantityBy(offset: number): BasketItemBuilder;
    /**
       * Set the measurement of this item to support items measured in fractions with a unit, such as "1.25 kilograms".
       *
       * The quantity (via [[withQuantity]] defaults to 1, but can also be set to represent multiple items, such as "2 x 1.25 kilograms".
       *
       * See [[withFractionalQuantity]] for a convenience function that sets quantity and measurement as per provided parameters.
       *
       * @param value The measurement value
       * @param unit  The unit (such as "kilograms" or "feet")
       * @return This builder
       */
    withMeasurement(value: number, unit: string): BasketItemBuilder;
    /**
       * Convenience method for scenarios where the quantity is represented as float/double internally in the client application.
       *
       * If a unit is set, the provided values will be set as per [[withMeasurement]] and the quantity defaults to 1.
       *
       * If no unit is set (null) and the provided quantity is a whole number, the value will be set as per [[withQuantity]].
       *
       * If no unit is set and the provided quantity has a fractional part, an exception will be thrown as unit is mandatory for measurement.
       *
       * @param quantity The floating point quantity
       * @param unit     The unit (such as "kilograms" or "feet") - may be undefined
       * @return This builder
       * @throws IllegalArgumentException When the unit is not set for a quantity with a fractional part
       */
    withFractionalQuantity(quantity: number, unit?: string): BasketItemBuilder;
    private hasFractionalPart;
    /**
       * Set the label for this item.
       *
       * @param label The label
       * @return This builder
       */
    withLabel(label: string): BasketItemBuilder;
    /**
       * Set the category for this item (such as "drinks" or "vegetables")
       *
       * @param category The category the item belongs to
       * @return This builder
       */
    withCategory(category: string): BasketItemBuilder;
    /**
       * Set the item amount value, inclusive of tax.
       *
       * This is to be used for cases where modifiers (see [[withBaseAmountAndModifiers]] are not relevant and a client
       * just wants to set the absolute amount value of this item.
       *
       * Note that the amount value can be negative to represent discounts, etc.
       *
       * See [[withBaseAmountAndModifiers]] to instead provide a base amount with modifiers applied.
       *
       * @param amount The item amount value, inclusive of tax.
       * @return This builder
       */
    withAmount(amount: number): BasketItemBuilder;
    /**
       * Set the item base amount value together with any modifiers that are to be applied to this amount.
       *
       * [[withAmount]] may NOT be called in combination with this - the amount field is in this case calculated
       * from the base amount with modifiers applied, and then rounded.
       *
       * @param baseAmount          The base amount value (exclusive of modifiers)
       * @param basketItemModifiers The var-args list of modifiers to apply to base amount
       * @return This builder
       */
    withBaseAmountAndModifiers(baseAmount: number, ...basketItemModifiers: BasketItemModifier[]): BasketItemBuilder;
    /**
       * Add additional item data entries.
       *
       * @param key    The key to use for this data
       * @param values An array of values for this data
       * @param <T>    The type of object this data is an array of
       * @return This builder
       */
    withItemData(key: string, ...values: any): BasketItemBuilder;
    /**
       * Add additional item data.
       *
       * @param additionalData The additional item data
       * @return This builder
       */
    withAdditionalData(additionalData: AdditionalData): BasketItemBuilder;
    /**
       * Build the instance with a default quantity of 1 (if not set).
       *
       * @return A {@link BasketItem} instance
       */
    build(): BasketItem;
}
