import { Measurement } from './measurement';
import { BasketItemModifier } from './basket-item-modifier';
import { AdditionalData } from './additional-data';
/**
 * Represents an immutable basket item with associated id, label, category, quantity (and optionally measurement) and amount.
 *
 * A basket item may have a positive or negative amount, representing both purchases and applied discounts.
 *
 * Additional information such as tax, add-ons, etc can be added as modifiers and other arbitrary data as item data.
 *
 * Please create instances via {@link BasketItemBuilder}.
 */
export declare class BasketItem {
    id: string;
    label: string;
    category?: string;
    amount: number;
    baseAmount: number;
    quantity: number;
    measurement?: Measurement;
    modifiers?: Array<BasketItemModifier>;
    itemData: AdditionalData;
    /**
       * Create a new basket item with label, category, amount (inclusive of tax) and quantity.
       *
       * @param id          The identifier (SKU or similar) for this item
       * @param label       The label of the item to show to merchants/customers, such as "Red onion"
       * @param category    The category the item belongs to, such as "vegetables" or "dairy"
       * @param amount      The amount (cost, rounded) for this (individual) item, inclusive of modifiers and tax
       * @param baseAmount  The base amount (cost) for this (individual) item, exclusive of modifiers
       * @param quantity    The quantity (count) of this basket item (default is 1, below 0 will produce an exception)
       * @param measurement The measurement of this basket item (for items that are measured in fractions and require a unit)
       * @param modifiers   The modifiers for the basket item
       * @param itemData    The item additional data
       *
       * @returns The new BasketItem instance
       */
    static from(id: string, label: string, category: string, amount: number, baseAmount: number, quantity: number, measurement?: Measurement, modifiers?: Array<BasketItemModifier>, itemData?: AdditionalData): BasketItem;
    /**
       * Get the total base cost (amount) for the items of this type, exclusive of any modifiers.
       *
       * Note that the amount may be negative in the case of discounts, etc.
       *
       * @return The total cost (amount) for the items of this type, excluding modifiers
       */
    getTotalBaseAmount(): number;
    /**
       * Get the total cost (amount) for the items of this type, inclusive of modifiers.
       *
       * This value has been rounded to closest whole sub-unit if the calculated amount has decimal points.
       *
       * Note that the amount may be negative in the case of discounts, etc.
       *
       * @return The total cost (amount) for the items of this type, including modifiers
       */
    getTotalAmount(): number;
    /**
       * Get the total cost (amount) for the items of this type, calculated from the base amount with modifiers applied.
       *
       * This can be used when rounding does not provide enough accuracy for calculating the total basket value.
       *
       * @return The total fractional amount, calculated from base amount with modifiers applied
       */
    getTotalFractionalAmount(): number;
    /**
       * Check whether this item has any modifiers.
       *
       * @return True if there are modifiers, false otherwise
       */
    hasModifiers(): boolean;
    /**
       * Check whether this item has a measurement value
       *
       * @return True if this item is measured
       */
    hasMeasurement(): boolean;
    /**
       * Check whether this basket item has any associated item data.
       *
       * @return True if there is any item data, false otherwise
       */
    hasItemData(): boolean;
    /**
       * Calculate the final amount after applying modifiers to the base amount.
       *
       * @param baseAmount The base amount value
       * @param modifiers  The list of modifiers
       * @return The amount value calculated from base with modifiers applied
       */
    static calculateFinalAmount(baseAmount: number, modifiers: Array<BasketItemModifier> | undefined): number;
}
