"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const rxjs_1 = require("rxjs");
const flow_config_1 = require("./flow-config");
let FlowConfigurations = class FlowConfigurations {
    constructor() {
        this.flowConfigurations = [];
    }
    /**
       * Get the flow configuration with the provided name.
       *
       * @param flowName The flow name
       * @return The flow config
       */
    getFlowConfiguration(flowName) {
        return this.fromName(flowName);
    }
    /**
       * Stream the list of flow configurations for simple filtering, conversions, etc.
       *
       * @return An Observable stream of {@link FlowConfig}
       */
    stream() {
        return rxjs_1.from(this.flowConfigurations);
    }
    /**
       * Get supported flow types for the given request class.
       *
       * If this should return flow types for initiating generic {@link com.aevi.sdk.flow.model.Request}, then use {@link FlowConfig#REQUEST_CLASS_GENERIC}
       *
       * If this should return flow types for initiating {@link com.aevi.sdk.pos.flow.model.Payment}, then use {@link FlowConfig#REQUEST_CLASS_PAYMENT}
       *
       * If null is passed, all types will be returned.
       *
       * @param requestClass {@link FlowConfig#REQUEST_CLASS_GENERIC}, {@link FlowConfig#REQUEST_CLASS_PAYMENT} or null for all types
       * @return A list of supported flow types
       */
    getFlowTypes(requestClass) {
        const flowTypes = new Array();
        for (const flowConfig of this.flowConfigurations) {
            if (requestClass == null || flowConfig.getRequestClass() === requestClass) {
                flowTypes.push(flowConfig.type);
            }
        }
        return flowTypes;
    }
    /**
       * Check whether a flow type is supported or not.
       *
       * A flow type is defined as supported if there is at least one flow configuration defined for that type.
       *
       * @param type The flow type to check
       * @return True if there is at least one flow for this type, false otherwise
       */
    isFlowTypeSupported(type) {
        for (const flowConfig of this.flowConfigurations) {
            if (flowConfig.type === type) {
                return true;
            }
        }
        return false;
    }
    /**
       * Get a list of all the flow names that are associated with the provided types.
       *
       * @param typesArray The types to filter by
       * @return The list of flow names
       */
    getFlowNamesForType(...typesArray) {
        const flowNames = new Array();
        for (const flowConfiguration of this.flowConfigurations) {
            if (typesArray.find((x) => x === flowConfiguration.type)) {
                flowNames.push(flowConfiguration.name);
            }
        }
        return flowNames;
    }
    /**
       * Get a list of all the flow configs that are associated with the provided types.
       *
       * @param typesArray The types to filter by
       * @return The list of flow names
       */
    getFlowConfigsForType(...typesArray) {
        const flowConfigs = new Array();
        for (const flowConfiguration of this.flowConfigurations) {
            if (typesArray.find((x) => x === flowConfiguration.type)) {
                flowConfigs.push(flowConfiguration);
            }
        }
        return flowConfigs;
    }
    /**
       * Check whether a particular flow has the provided stage defined.
       *
       * @param stage    The flow stage
       * @param flowName The flow to check if the stage is defined for
       * @return True if the flow has the stage defined, false otherwise
       */
    isStageDefinedForFlow(stage, flowName) {
        const flowConfig = this.fromName(flowName);
        if (flowConfig != null) {
            return flowConfig.hasStage(stage);
        }
        return false;
    }
    fromName(flowName) {
        for (const flowConfiguration of this.flowConfigurations) {
            if (flowConfiguration.name === flowName) {
                return flowConfiguration;
            }
        }
        return null;
    }
};
__decorate([
    json2typescript_1.JsonProperty('flowConfigurations', [flow_config_1.FlowConfig]),
    __metadata("design:type", Array)
], FlowConfigurations.prototype, "flowConfigurations", void 0);
FlowConfigurations = __decorate([
    json2typescript_1.JsonObject('FlowConfigurations')
], FlowConfigurations);
exports.FlowConfigurations = FlowConfigurations;
