import { Amounts } from './amounts';
import { Basket } from './basket';
import { Customer } from './customer';
import { AdditionalData } from './additional-data';
import { TransactionResponse } from './transaction-response';
import { FlowAppInfo } from './flow-app-info';
/**
 * Represents a transaction within a flow.
 *
 * In the case of a split enabled flow, there will be an instance of this class for each "split", which usually represents each customer.
 *
 * {@link TransactionRequest} instances are created as required to call flow services within this transaction, with the remaining amounts to process.
 *
 * A transaction can contain zero to many {@link TransactionResponse} instances, as a result of calling into flow services that pay off a portion
 * or all of the requested amounts.
 *
 * Use [[requestedAmounts]] to retrieve the total amount requested for this transaction, and [[getRemainingAmounts]] to retrieve
 * the amounts remaining to pay for this transaction, if any.
 */
export declare class Transaction {
    requestedAmounts: Amounts;
    baskets?: Array<Basket>;
    customer?: Customer;
    additionalData?: AdditionalData;
    transactionResponses: Array<TransactionResponse>;
    executedFlowApps: Array<FlowAppInfo>;
    /**
       * Get the remaining amounts required to fulfill the requested amounts for this transaction.
       *
       * This shall be used to determine how much to charge the customer at any given point.
       *
       * @return The remaining amounts.
       */
    getRemainingAmounts(): Amounts;
    /**
       * Get the amounts processed at this point in time.
       *
       * Note that the processed amount may be larger than the requested amounts.
       *
       * @return The amounts processed at this point in time.
       */
    getProcessedAmounts(): Amounts;
    /**
       * Check whether the requested amounts have been fully processed.
       *
       * @return True if the amounts have been fully processed, false otherwise.
       */
    hasProcessedRequestedAmounts(): boolean;
    /**
       * Check whether this transaction has any responses or not yet.
       *
       * @return True if there are transaction responses, false otherwise.
       */
    hasResponses(): boolean;
    /**
       * Check whether there were any declined responses.
       *
       * @return True if there were declined responses, false otherwise
       */
    hasDeclinedResponses(): boolean;
    /**
       * Get the last transaction response, if any.
       *
       * @return The last transaction response, if one is available.
       */
    getLastResponse(): TransactionResponse;
    /**
       * Get the transaction response generated by a payment application specifically, if any.
       *
       * This is a convenience method that can be used when the response details such as id is required to initiate follow-up requests against the
       * payment app, such as reversals.
       *
       * @return The transaction response generated by a payment application if set, or null
       */
    getPaymentAppResponse(): TransactionResponse;
}
