"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const amounts_1 = require("./amounts");
const basket_1 = require("./basket");
const customer_1 = require("./customer");
const additional_data_1 = require("./additional-data");
const transaction_response_1 = require("./transaction-response");
const flow_app_info_1 = require("./flow-app-info");
const constants_1 = require("./constants");
/**
 * Represents a transaction within a flow.
 *
 * In the case of a split enabled flow, there will be an instance of this class for each "split", which usually represents each customer.
 *
 * {@link TransactionRequest} instances are created as required to call flow services within this transaction, with the remaining amounts to process.
 *
 * A transaction can contain zero to many {@link TransactionResponse} instances, as a result of calling into flow services that pay off a portion
 * or all of the requested amounts.
 *
 * Use [[requestedAmounts]] to retrieve the total amount requested for this transaction, and [[getRemainingAmounts]] to retrieve
 * the amounts remaining to pay for this transaction, if any.
 */
let Transaction = class Transaction {
    constructor() {
        this.requestedAmounts = undefined;
        this.baskets = undefined;
        this.customer = undefined;
        this.additionalData = undefined;
        this.transactionResponses = undefined;
        this.executedFlowApps = undefined;
    }
    /**
       * Get the remaining amounts required to fulfill the requested amounts for this transaction.
       *
       * This shall be used to determine how much to charge the customer at any given point.
       *
       * @return The remaining amounts.
       */
    getRemainingAmounts() {
        let remaining = amounts_1.Amounts.fromAmounts(this.requestedAmounts);
        for (const transactionResponse of this.transactionResponses) {
            if (transactionResponse.outcome === transaction_response_1.TransactionResponseOutcome.APPROVED && transactionResponse.amounts != null) {
                remaining = amounts_1.Amounts.subtractAmounts(remaining, transactionResponse.amounts, false);
            }
        }
        return remaining;
    }
    /**
       * Get the amounts processed at this point in time.
       *
       * Note that the processed amount may be larger than the requested amounts.
       *
       * @return The amounts processed at this point in time.
       */
    getProcessedAmounts() {
        let processed = amounts_1.Amounts.from(0, this.requestedAmounts.currency);
        for (const transactionResponse of this.transactionResponses) {
            if (transactionResponse.outcome === transaction_response_1.TransactionResponseOutcome.APPROVED && transactionResponse.amounts) {
                processed = amounts_1.Amounts.addAmounts(processed, transactionResponse.amounts);
            }
        }
        return processed;
    }
    /**
       * Check whether the requested amounts have been fully processed.
       *
       * @return True if the amounts have been fully processed, false otherwise.
       */
    hasProcessedRequestedAmounts() {
        return this.getRemainingAmounts().getTotalAmountValue() === 0;
    }
    /**
       * Check whether this transaction has any responses or not yet.
       *
       * @return True if there are transaction responses, false otherwise.
       */
    hasResponses() {
        return this.transactionResponses.length > 0;
    }
    /**
       * Check whether there were any declined responses.
       *
       * @return True if there were declined responses, false otherwise
       */
    hasDeclinedResponses() {
        for (const transactionResponse of this.transactionResponses) {
            if (transactionResponse.outcome === transaction_response_1.TransactionResponseOutcome.DECLINED) {
                return true;
            }
        }
        return false;
    }
    /**
       * Get the last transaction response, if any.
       *
       * @return The last transaction response, if one is available.
       */
    getLastResponse() {
        if (this.transactionResponses.length > 0) {
            return this.transactionResponses[this.transactionResponses.length - 1];
        }
        return null;
    }
    /**
       * Get the transaction response generated by a payment application specifically, if any.
       *
       * This is a convenience method that can be used when the response details such as id is required to initiate follow-up requests against the
       * payment app, such as reversals.
       *
       * @return The transaction response generated by a payment application if set, or null
       */
    getPaymentAppResponse() {
        for (const transactionResponse of this.transactionResponses) {
            if (transactionResponse.flowStage && (transactionResponse.flowStage === constants_1.FlowStages.PAYMENT_CARD_READING
                || transactionResponse.flowStage === constants_1.FlowStages.TRANSACTION_PROCESSING)) {
                return transactionResponse;
            }
        }
        return null;
    }
};
__decorate([
    json2typescript_1.JsonProperty('requestedAmounts', amounts_1.Amounts),
    __metadata("design:type", amounts_1.Amounts)
], Transaction.prototype, "requestedAmounts", void 0);
__decorate([
    json2typescript_1.JsonProperty('baskets', [basket_1.Basket], true),
    __metadata("design:type", Array)
], Transaction.prototype, "baskets", void 0);
__decorate([
    json2typescript_1.JsonProperty('customer', customer_1.Customer, true),
    __metadata("design:type", customer_1.Customer)
], Transaction.prototype, "customer", void 0);
__decorate([
    json2typescript_1.JsonProperty('additionalData', additional_data_1.AdditionalData),
    __metadata("design:type", additional_data_1.AdditionalData)
], Transaction.prototype, "additionalData", void 0);
__decorate([
    json2typescript_1.JsonProperty('transactionResponses', [transaction_response_1.TransactionResponse]),
    __metadata("design:type", Array)
], Transaction.prototype, "transactionResponses", void 0);
__decorate([
    json2typescript_1.JsonProperty('executedFlowApps', [flow_app_info_1.FlowAppInfo]),
    __metadata("design:type", Array)
], Transaction.prototype, "executedFlowApps", void 0);
Transaction = __decorate([
    json2typescript_1.JsonObject('Transaction')
], Transaction);
exports.Transaction = Transaction;
