import { Amount } from './amount';
import { Jsonable } from './jsonable';
/**
 * Representation of all the amounts relevant for a transaction.
 *
 * See {@link Amount} for representation of a single amount value with associated currency.
 */
export declare class Amounts extends Jsonable {
    baseAmount: number;
    additionalAmounts: {
        [name: string]: number;
    };
    currency: string;
    currencyExchangeRate: number;
    originalCurrency?: string;
    /**
       * Convert a JSON string into an {@link Amounts} object if possible
       *
       * @param json The JSON to convert
       */
    static fromJson(json: string): Amounts;
    /**
       * Initialise with base amount (inclusive of tax), currency and optional additional amounts map.
       *
       * @param baseAmount        The base amount, inclusive of tax, in subunit form (cents, pence, etc)
       * @param currency          The ISO-4217 currency code
       * @param additionalAmounts The additional amounts
       *
       * @returns The initialised amounts object
       */
    static from(baseAmount?: number, currency?: string, additionalAmounts?: {}): Amounts;
    /**
       * Initialise a new Amounts object from another one.
       *
       * The method will create a new Amounts object cloned from the original
       *
       * @param from The Amounts object to clone
       *
       * @returns The initialised amounts object
       */
    static fromAmounts(from: Amounts): Amounts;
    /**
       * Add an additional amount to complement the base amount.
       *
       * The additional amounts are represented via a string identifier and the amount value in subunit form.
       *
       * Examples of identifiers are "tip" and "cashback". Note that identifiers are case sensitive!
       *
       * @param identifier The string identifier for the amount
       * @param amount     The amount value
       */
    addAdditionalAmount(identifier: string, amount: number): void;
    /**
       * Add an additional amount as a fraction of the base amount.
       *
       * This is useful for cases where a fee, charity contribution, etc is calculated as a fraction or percentage of the base amount value.
       *
       * Examples of identifiers are "tip" and "cashback". Note that identifiers are case sensitive!
       *
       * @param identifier The string identifier for the amount
       * @param fraction   The fraction of the base amount, ranging from 0.0 to 1.0f (0% to 100%)
       */
    addAdditionalAmountAsBaseFraction(identifier: string, fraction: number): void;
    /**
       * Get an {@link Amount} representation of the base amount with associated currency.
       *
       * The base amount is inclusive of any tax.
       *
       * @return Base {@link Amount}
       */
    getBaseAmount(): Amount;
    /**
       * Check whether there is an additional amount with the provided identifier defined.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return True if one is defined, false otherwise
       */
    hasAdditionalAmount(identifier: string): boolean;
    /**
       * Get the additional amount value for the provided identifier.
       *
       * If none is set, 0 will be returned.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return The amount value
       */
    getAdditionalAmountValue(identifier: string): number;
    /**
       * Get an {@link Amount} representation of the additional amount with associated currency.
       *
       * Note that identifiers are case sensitive.
       *
       * @param identifier The identifier
       * @return The additional {@link Amount}
       */
    getAdditionalAmount(identifier: string): Amount;
    /**
       * Get the map of all the additional amounts set.
       *
       * Note that amount identifiers are case sensitive.
       *
       * @return The map of identifier keys mapped to amount values
       */
    getAdditionalAmounts(): {
        [name: string]: number;
    };
    /**
       * Get the total amount (base + additional amounts) in subunit form.
       *
       * @return The total amount
       */
    getTotalAmountValue(): number;
    /**
       * Get the total amount (base + additional amounts) in subunit form, excluding any additionals as defined by their provided identifiers.
       *
       * This is useful for scenarios where some additional amounts are supported natively in an environment (such as tip and cashback), but others
       * (like charity donations) are not and should be appended to the base amount for that environment.
       *
       * Note that identifiers are case sensitive.
       *
       * @param amountIdentifiers The identifiers of the amounts to exclude from the calculation
       * @return The total amount value, excluding the amounts as provider via the identifiers
       */
    getTotalExcludingAmounts(...amountIdentifiers: Array<string>): number;
    /**
       * Get an {@link Amount} representation of the total amount with associated currency.
       *
       * @return Total {@link Amount}
       */
    getTotalAmount(): Amount;
    /**
       * Add together two amounts and get a new Amounts instance back with the result.
       *
       * @param a1 The first amount to add with
       * @param a2 The second amount to add with
       * @return The combined result
       */
    static addAmounts(a1: Amounts, a2: Amounts): Amounts;
    /**
       * Subtract one amounts from another.
       *
       * Note that the result Amounts will only contain additionalAmounts defined in a1, the one being subtracted from.
       * If an amount is set in a2 only, it will not be added to the resulting Amounts.
       *
       * @param a1                        The amounts to subtract a2 from
       * @param a2                        The amounts of which a1 will be reduced by
       * @param keepZeroAmountAdditionals Whether or not to keep additional amounts with a value of zero
       * @return The reduced amounts
       */
    static subtractAmounts(a1: Amounts, a2: Amounts, keepZeroAmountAdditionals?: boolean): Amounts;
}
