"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var BasketItem_1;
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const measurement_1 = require("./measurement");
const basket_item_modifier_1 = require("./basket-item-modifier");
const additional_data_1 = require("./additional-data");
/**
 * Represents an immutable basket item with associated id, label, category, quantity (and optionally measurement) and amount.
 *
 * A basket item may have a positive or negative amount, representing both purchases and applied discounts.
 *
 * Additional information such as tax, add-ons, etc can be added as modifiers and other arbitrary data as item data.
 *
 * Please create instances via {@link BasketItemBuilder}.
 */
let BasketItem = BasketItem_1 = class BasketItem {
    constructor() {
        this.id = undefined;
        this.label = undefined;
        this.category = undefined;
        this.amount = undefined;
        this.baseAmount = undefined;
        this.quantity = undefined;
        this.measurement = undefined;
        this.modifiers = undefined;
        this.itemData = new additional_data_1.AdditionalData();
    }
    /**
       * Create a new basket item with label, category, amount (inclusive of tax) and quantity.
       *
       * @param id          The identifier (SKU or similar) for this item
       * @param label       The label of the item to show to merchants/customers, such as "Red onion"
       * @param category    The category the item belongs to, such as "vegetables" or "dairy"
       * @param amount      The amount (cost, rounded) for this (individual) item, inclusive of modifiers and tax
       * @param baseAmount  The base amount (cost) for this (individual) item, exclusive of modifiers
       * @param quantity    The quantity (count) of this basket item (default is 1, below 0 will produce an exception)
       * @param measurement The measurement of this basket item (for items that are measured in fractions and require a unit)
       * @param modifiers   The modifiers for the basket item
       * @param itemData    The item additional data
       *
       * @returns The new BasketItem instance
       */
    static from(id, label, category, amount, baseAmount, quantity, measurement, modifiers, itemData) {
        const basketItem = new BasketItem_1();
        basketItem.id = id;
        basketItem.label = label;
        basketItem.category = category;
        basketItem.amount = amount;
        basketItem.baseAmount = baseAmount;
        basketItem.quantity = quantity;
        basketItem.measurement = measurement;
        basketItem.modifiers = modifiers;
        if (itemData) {
            basketItem.itemData = itemData;
        }
        return basketItem;
    }
    /**
       * Get the total base cost (amount) for the items of this type, exclusive of any modifiers.
       *
       * Note that the amount may be negative in the case of discounts, etc.
       *
       * @return The total cost (amount) for the items of this type, excluding modifiers
       */
    getTotalBaseAmount() {
        return this.baseAmount * this.quantity;
    }
    /**
       * Get the total cost (amount) for the items of this type, inclusive of modifiers.
       *
       * This value has been rounded to closest whole sub-unit if the calculated amount has decimal points.
       *
       * Note that the amount may be negative in the case of discounts, etc.
       *
       * @return The total cost (amount) for the items of this type, including modifiers
       */
    getTotalAmount() {
        return this.amount * this.quantity;
    }
    /**
       * Get the total cost (amount) for the items of this type, calculated from the base amount with modifiers applied.
       *
       * This can be used when rounding does not provide enough accuracy for calculating the total basket value.
       *
       * @return The total fractional amount, calculated from base amount with modifiers applied
       */
    getTotalFractionalAmount() {
        if (!this.hasModifiers()) {
            return this.getTotalAmount();
        }
        const amount = BasketItem_1.calculateFinalAmount(this.baseAmount, this.modifiers);
        return amount * this.quantity;
    }
    /**
       * Check whether this item has any modifiers.
       *
       * @return True if there are modifiers, false otherwise
       */
    hasModifiers() {
        return this.modifiers && this.modifiers.length > 0;
    }
    /**
       * Check whether this item has a measurement value
       *
       * @return True if this item is measured
       */
    hasMeasurement() {
        return this.measurement != null && this.measurement !== undefined;
    }
    /**
       * Check whether this basket item has any associated item data.
       *
       * @return True if there is any item data, false otherwise
       */
    hasItemData() {
        return this.itemData != null && this.itemData !== undefined && !this.itemData.isEmpty();
    }
    /**
       * Calculate the final amount after applying modifiers to the base amount.
       *
       * @param baseAmount The base amount value
       * @param modifiers  The list of modifiers
       * @return The amount value calculated from base with modifiers applied
       */
    static calculateFinalAmount(baseAmount, modifiers) {
        let amount = baseAmount;
        if (modifiers) {
            for (const modifier of modifiers) {
                if (modifier.amount != null && modifier.amount !== 0.0) {
                    amount += modifier.amount;
                }
                else if (modifier.percentage != null) {
                    amount += baseAmount * (modifier.percentage / 100.0);
                }
            }
        }
        return amount;
    }
};
__decorate([
    json2typescript_1.JsonProperty('id'),
    __metadata("design:type", String)
], BasketItem.prototype, "id", void 0);
__decorate([
    json2typescript_1.JsonProperty('label'),
    __metadata("design:type", String)
], BasketItem.prototype, "label", void 0);
__decorate([
    json2typescript_1.JsonProperty('category', String, true),
    __metadata("design:type", String)
], BasketItem.prototype, "category", void 0);
__decorate([
    json2typescript_1.JsonProperty('amount'),
    __metadata("design:type", Number)
], BasketItem.prototype, "amount", void 0);
__decorate([
    json2typescript_1.JsonProperty('baseAmount'),
    __metadata("design:type", Number)
], BasketItem.prototype, "baseAmount", void 0);
__decorate([
    json2typescript_1.JsonProperty('quantity'),
    __metadata("design:type", Number)
], BasketItem.prototype, "quantity", void 0);
__decorate([
    json2typescript_1.JsonProperty('measurement', measurement_1.Measurement, true),
    __metadata("design:type", measurement_1.Measurement)
], BasketItem.prototype, "measurement", void 0);
__decorate([
    json2typescript_1.JsonProperty('modifiers', [basket_item_modifier_1.BasketItemModifier], true),
    __metadata("design:type", Array)
], BasketItem.prototype, "modifiers", void 0);
__decorate([
    json2typescript_1.JsonProperty('itemData', additional_data_1.AdditionalData, true),
    __metadata("design:type", Object)
], BasketItem.prototype, "itemData", void 0);
BasketItem = BasketItem_1 = __decorate([
    json2typescript_1.JsonObject('BasketItem')
], BasketItem);
exports.BasketItem = BasketItem;
