import { Jsonable } from './jsonable';
import { BasketItem } from './basket-item';
import { AdditionalData } from './additional-data';
/**
 * Strategies for rounding decimal numbers.
 */
export declare enum RoundingStrategy {
    /**
       * Always round down. Both 15.2 and 15.9 is rounded to 15.
       */
    DOWN = "DOWN",
    /**
       * Round to nearest. 15.2 is rounded to 15, and 15.9 is rounded to 16.
       */
    NEAREST = "NEAREST",
    /**
       * Always round up. Both 15.2 and 15.9 is rounded to 16.
       */
    UP = "UP"
}
/**
 * Represents a basket consisting of one or multiple {@link BasketItem}.
 *
 * Baskets are uniquely identified via a randomly generated id, and also contains a name as a readable identifier.
 *
 * Basket items are kept in a list sorted by most recently added first.
 *
 * Basket items are uniquely identified by an id, meaning it is possible that there is more than one item with the same label.
 * It is up to the client to manage this correctly.
 *
 * Note that as basket items are immutable, any update to items (such as merging or changing quantity) leads to new instances being created. For the
 * latest up to date item, always fetch via [[getItemById]].
 */
export declare class Basket extends Jsonable {
    basketName: string;
    displayItems: BasketItem[];
    additionalBasketData: AdditionalData;
    primaryBasket: boolean;
    roundingStrategy: RoundingStrategy;
    /**
       * Convert a JSON string into an {@link Basket} object if possible
       *
       * @param json The JSON to convert
       */
    static fromJson(json: string): Basket;
    /**
       * Initialise a basket from the provided var-args items, maintaining the same order as they are specified in.
       *
       * @param basketName  The name of the basket
       * @param basketItems The var-args list of basket items to initialise the basket with
       *
       * @returns The new Basket object
       */
    static fromItems(name: string, ...basketItems: Array<BasketItem>): Basket;
    /**
       * Get the total basket value, inclusive of tax.
       *
       * This value is calculated as the sum of each item base amount with modifiers applied and then rounded as per the strategy set
       * via [[setRoundingStrategy]].
       *
       * @return The total basket value
       */
    getTotalBasketValue(): number;
    /**
       * Add one or multiple basket items to the *front* of the basket either as a new item or via merging with an existing one with the same id.
       *
       * If there is an existing item with the same id in the basket, then the item quantity of the two items will be added together and stored.
       *
       * Note that due to how items may be merged, the instance passed into this method is not necessarily the same instance that is stored.
       *
       * @param items The item(s) to add
       */
    addItems(...items: Array<BasketItem>): void;
    /**
       * Check whether the basket has an item with the provided id.
       *
       * Optionally, a min quantity parameter can be passed in to filter results against. If an item with the id is found and the min quantity is set,
       * this method will only return true if the item has at least the min quantity.
       *
       * @param id          The id to match against
       * @param minQuantity Optional param to specify a minimum quantity criteria
       * @return True if there is an item with matching id, false otherwise
       */
    hasItemWithId(id: string, ...minQuantity: number[]): boolean;
    /**
       * Check whether the basket has an item with the provided label.
       *
       * Note that there may be more than one item with the same label.
       *
       * @param label The label to match against
       * @return True if there is an item with matching label, false otherwise.
       */
    hasItemWithLabel(label: string): boolean;
    /**
       * Get an item based on its id.
       *
       * @param id The id to match against
       * @return The item if found, or null
       */
    getItemById(id: string): BasketItem | null;
    /**
       * Get an item that matches the provided label.
       *
       * Note that there may be more than one item with the same label, in which case it returns the first one found.
       *
       * @param label The label to match against
       * @return The item if found, or null
       */
    getItemByLabel(label: string): BasketItem | null;
    /**
       * Get all items that belongs to the provided category.
       *
       * @param category The category to filter against
       * @return A list of items belonging to the provided category. May be empty.
       */
    getBasketItemsByCategory(category: string): Array<BasketItem>;
    /**
       * Increment the basket item quantity of the item with the provided id.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * @param itemId    The basket item id to increment the quantity of
       * @param increment The increment value (positive)
       * @return The item with updated quantity or null if no item with id found
       */
    incrementItemQuantity(itemId: string, increment: number): BasketItem | null;
    /**
       * Decrement the basket item quantity of the item with the provided id.
       *
       * The default behaviour is that items that are decremented to zero are removed from the basket. This can be overriden by the retainIfZero param.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * @param itemId       The basket item id to decrement the quantity of
       * @param decrement    The decrement value (positive)
       * @param retainIfZero If set to true, the item will be kept in the basket despite the quantity being zero
       * @return The item with the updated quantity or null if no item with id found or it is removed
       */
    decrementItemQuantity(itemId: string, decrement: number, ...retainIfZero: boolean[]): BasketItem | null;
    /**
       * Explicitly set the quantity for a basket item.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * The quantity must be zero or larger. Negative values are ignored.
       *
       * @param itemId      The item id
       * @param newQuantity The new quantity for the basket item (must be positive)
       * @return The item with the updated quantity or null if no item with id found
       */
    setItemQuantity(itemId: string, newQuantity: number): BasketItem | null;
    /**
       * Remove the item with the provided id.
       *
       * @param itemId The id of the item to remove
       * @return The basket item that was removed, or null
       */
    removeItem(itemId: string): BasketItem | null;
    /**
       * Clear the basket of all items.
       */
    clearItems(): void;
    /**
       * Get the number of unique items in the basket, incl any zero-quantity items.
       *
       * See [[getTotalNumberOfItems]] for retrieving the total number of items.
       *
       * @return The number of unique items in the basket
       */
    getNumberOfUniqueItems(): number;
    /**
       * Get the total number of items, taking into account the quantity of each individual item. This excludes zero-quantity items.
       *
       * See [[getNumberOfUniqueItems]] for retrieving the number of unique items.
       *
       * @return The total number of items
       */
    getTotalNumberOfItems(): number;
    /**
       * Add additional data to this basket.
       *
       * See [[addData]] for more info.
       *
       * @param key    The key to use for this data
       * @param values An array of values for this data
       * @param <T>    The type of object this data is an array of
       */
    addAdditionalData<T>(key: string, ...values: T[]): void;
    private replaceItem;
}
