"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Basket_1;
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const jsonable_1 = require("./jsonable");
const basket_item_1 = require("./basket-item");
const additional_data_1 = require("./additional-data");
const basket_item_builder_1 = require("./basket-item-builder");
/**
 * Strategies for rounding decimal numbers.
 */
var RoundingStrategy;
(function (RoundingStrategy) {
    /**
       * Always round down. Both 15.2 and 15.9 is rounded to 15.
       */
    RoundingStrategy["DOWN"] = "DOWN";
    /**
       * Round to nearest. 15.2 is rounded to 15, and 15.9 is rounded to 16.
       */
    RoundingStrategy["NEAREST"] = "NEAREST";
    /**
       * Always round up. Both 15.2 and 15.9 is rounded to 16.
       */
    RoundingStrategy["UP"] = "UP";
})(RoundingStrategy = exports.RoundingStrategy || (exports.RoundingStrategy = {}));
/**
 * Represents a basket consisting of one or multiple {@link BasketItem}.
 *
 * Baskets are uniquely identified via a randomly generated id, and also contains a name as a readable identifier.
 *
 * Basket items are kept in a list sorted by most recently added first.
 *
 * Basket items are uniquely identified by an id, meaning it is possible that there is more than one item with the same label.
 * It is up to the client to manage this correctly.
 *
 * Note that as basket items are immutable, any update to items (such as merging or changing quantity) leads to new instances being created. For the
 * latest up to date item, always fetch via [[getItemById]].
 */
let Basket = Basket_1 = class Basket extends jsonable_1.Jsonable {
    constructor() {
        super(...arguments);
        this.basketName = '';
        this.displayItems = [];
        this.additionalBasketData = new additional_data_1.AdditionalData();
        this.primaryBasket = false;
        this.roundingStrategy = RoundingStrategy.NEAREST;
    }
    /**
       * Convert a JSON string into an {@link Basket} object if possible
       *
       * @param json The JSON to convert
       */
    static fromJson(json) {
        return super.baseFromJson(json, Basket_1);
    }
    /**
       * Initialise a basket from the provided var-args items, maintaining the same order as they are specified in.
       *
       * @param basketName  The name of the basket
       * @param basketItems The var-args list of basket items to initialise the basket with
       *
       * @returns The new Basket object
       */
    static fromItems(name, ...basketItems) {
        const basket = new Basket_1();
        basket.basketName = name;
        basket.displayItems = basketItems;
        return basket;
    }
    /**
       * Get the total basket value, inclusive of tax.
       *
       * This value is calculated as the sum of each item base amount with modifiers applied and then rounded as per the strategy set
       * via [[setRoundingStrategy]].
       *
       * @return The total basket value
       */
    getTotalBasketValue() {
        let total = 0;
        for (const displayItem of this.displayItems) {
            total += displayItem.getTotalFractionalAmount();
        }
        if (this.roundingStrategy != null) {
            switch (this.roundingStrategy) {
                case RoundingStrategy.DOWN:
                    return Math.floor(total);
                case RoundingStrategy.UP:
                    return Math.ceil(total);
                case RoundingStrategy.NEAREST:
                default:
                    return Math.round(total);
            }
        }
        return Math.round(total);
    }
    /**
       * Add one or multiple basket items to the *front* of the basket either as a new item or via merging with an existing one with the same id.
       *
       * If there is an existing item with the same id in the basket, then the item quantity of the two items will be added together and stored.
       *
       * Note that due to how items may be merged, the instance passed into this method is not necessarily the same instance that is stored.
       *
       * @param items The item(s) to add
       */
    addItems(...items) {
        for (const item of items) {
            const existingItem = this.getItemById(item.id);
            if (existingItem != null) {
                this.replaceItem(existingItem, item.quantity, true);
            }
            else {
                this.displayItems.splice(0, 0, item);
            }
        }
    }
    /**
       * Check whether the basket has an item with the provided id.
       *
       * Optionally, a min quantity parameter can be passed in to filter results against. If an item with the id is found and the min quantity is set,
       * this method will only return true if the item has at least the min quantity.
       *
       * @param id          The id to match against
       * @param minQuantity Optional param to specify a minimum quantity criteria
       * @return True if there is an item with matching id, false otherwise
       */
    hasItemWithId(id, ...minQuantity) {
        const minQuantityValue = minQuantity.length > 0 ? minQuantity[0] : -1;
        for (const item of this.displayItems) {
            if (item.id === id && item.quantity >= minQuantityValue) {
                return true;
            }
        }
        return false;
    }
    /**
       * Check whether the basket has an item with the provided label.
       *
       * Note that there may be more than one item with the same label.
       *
       * @param label The label to match against
       * @return True if there is an item with matching label, false otherwise.
       */
    hasItemWithLabel(label) {
        for (const item of this.displayItems) {
            if (item.label === label) {
                return true;
            }
        }
        return false;
    }
    /**
       * Get an item based on its id.
       *
       * @param id The id to match against
       * @return The item if found, or null
       */
    getItemById(id) {
        for (const item of this.displayItems) {
            if (item.id === id) {
                return item;
            }
        }
        return null;
    }
    /**
       * Get an item that matches the provided label.
       *
       * Note that there may be more than one item with the same label, in which case it returns the first one found.
       *
       * @param label The label to match against
       * @return The item if found, or null
       */
    getItemByLabel(label) {
        for (const item of this.displayItems) {
            if (item.label === label) {
                return item;
            }
        }
        return null;
    }
    /**
       * Get all items that belongs to the provided category.
       *
       * @param category The category to filter against
       * @return A list of items belonging to the provided category. May be empty.
       */
    getBasketItemsByCategory(category) {
        const items = [];
        for (const displayItem of this.displayItems) {
            const itemCategory = displayItem.category;
            if (itemCategory != null && itemCategory === category) {
                items.push(displayItem);
            }
        }
        return items;
    }
    /**
       * Increment the basket item quantity of the item with the provided id.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * @param itemId    The basket item id to increment the quantity of
       * @param increment The increment value (positive)
       * @return The item with updated quantity or null if no item with id found
       */
    incrementItemQuantity(itemId, increment) {
        const item = this.getItemById(itemId);
        if (item != null) {
            return this.replaceItem(item, increment, true);
        }
        return null;
    }
    /**
       * Decrement the basket item quantity of the item with the provided id.
       *
       * The default behaviour is that items that are decremented to zero are removed from the basket. This can be overriden by the retainIfZero param.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * @param itemId       The basket item id to decrement the quantity of
       * @param decrement    The decrement value (positive)
       * @param retainIfZero If set to true, the item will be kept in the basket despite the quantity being zero
       * @return The item with the updated quantity or null if no item with id found or it is removed
       */
    decrementItemQuantity(itemId, decrement, ...retainIfZero) {
        const item = this.getItemById(itemId);
        if (item != null) {
            const retain = retainIfZero.length > 0 && retainIfZero[0];
            return this.replaceItem(item, -decrement, retain);
        }
        return null;
    }
    /**
       * Explicitly set the quantity for a basket item.
       *
       * Note that if no such item exists, this is a no-op.
       *
       * The quantity must be zero or larger. Negative values are ignored.
       *
       * @param itemId      The item id
       * @param newQuantity The new quantity for the basket item (must be positive)
       * @return The item with the updated quantity or null if no item with id found
       */
    setItemQuantity(itemId, newQuantity) {
        if (newQuantity >= 0) {
            const item = this.getItemById(itemId);
            if (item != null) {
                return this.replaceItem(item, newQuantity - item.quantity, true);
            }
        }
        return null;
    }
    /**
       * Remove the item with the provided id.
       *
       * @param itemId The id of the item to remove
       * @return The basket item that was removed, or null
       */
    removeItem(itemId) {
        const foundItem = this.getItemById(itemId);
        if (foundItem != null) {
            this.displayItems = this.displayItems.filter((item) => item.id !== itemId);
            return foundItem;
        }
        return null;
    }
    /**
       * Clear the basket of all items.
       */
    clearItems() {
        this.displayItems = [];
    }
    /**
       * Get the number of unique items in the basket, incl any zero-quantity items.
       *
       * See [[getTotalNumberOfItems]] for retrieving the total number of items.
       *
       * @return The number of unique items in the basket
       */
    getNumberOfUniqueItems() {
        return this.displayItems.length;
    }
    /**
       * Get the total number of items, taking into account the quantity of each individual item. This excludes zero-quantity items.
       *
       * See [[getNumberOfUniqueItems]] for retrieving the number of unique items.
       *
       * @return The total number of items
       */
    getTotalNumberOfItems() {
        let total = 0;
        for (const displayItem of this.displayItems) {
            total += displayItem.quantity;
        }
        return total;
    }
    /**
       * Add additional data to this basket.
       *
       * See [[addData]] for more info.
       *
       * @param key    The key to use for this data
       * @param values An array of values for this data
       * @param <T>    The type of object this data is an array of
       */
    addAdditionalData(key, ...values) {
        this.additionalBasketData.addData(key, values);
    }
    replaceItem(existingItem, quantityOffset, retainIfZero) {
        const newItem = basket_item_builder_1.BasketItemBuilder.from(existingItem).offsetQuantityBy(quantityOffset).build();
        if (newItem.quantity === 0 && !retainIfZero) {
            this.displayItems = this.displayItems.filter((item) => item.id !== existingItem.id);
        }
        else {
            this.displayItems[this.displayItems.indexOf(existingItem)] = newItem;
        }
        return newItem;
    }
};
__decorate([
    json2typescript_1.JsonProperty('basketName'),
    __metadata("design:type", String)
], Basket.prototype, "basketName", void 0);
__decorate([
    json2typescript_1.JsonProperty('displayItems', [basket_item_1.BasketItem]),
    __metadata("design:type", Array)
], Basket.prototype, "displayItems", void 0);
__decorate([
    json2typescript_1.JsonProperty('additionalBasketData', additional_data_1.AdditionalData, true),
    __metadata("design:type", additional_data_1.AdditionalData)
], Basket.prototype, "additionalBasketData", void 0);
__decorate([
    json2typescript_1.JsonProperty('primaryBasket'),
    __metadata("design:type", Boolean)
], Basket.prototype, "primaryBasket", void 0);
__decorate([
    json2typescript_1.JsonProperty('roundingStrategy', String),
    __metadata("design:type", String)
], Basket.prototype, "roundingStrategy", void 0);
Basket = Basket_1 = __decorate([
    json2typescript_1.JsonObject('Basket')
], Basket);
exports.Basket = Basket;
