"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Card_1;
Object.defineProperty(exports, "__esModule", { value: true });
/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
const json2typescript_1 = require("json2typescript");
const moment = require("moment");
const additional_data_1 = require("./additional-data");
const token_1 = require("./token");
const pre_conditions_1 = require("../util/pre-conditions");
/**
 * The card details representing the card presented during a transaction.
 *
 * All fields in this class are optional and the payment service used may or may not return all these values.
 */
let Card = Card_1 = class Card {
    constructor() {
        this.maskedPan = undefined;
        this.cardholderName = undefined;
        this.expiryDate = undefined;
        this.cardToken = undefined;
        this.additionalData = new additional_data_1.AdditionalData();
    }
    /**
       * Check whether this object contains any card data.
       *
       * @return True if empty, false if contains data
       */
    isEmpty() {
        return !this.maskedPan && !this.cardholderName && !this.cardToken && !this.expiryDate && this.additionalData.isEmpty();
    }
    /**
       * Create and returns a new Card object from the parameters given
       *
       * @param maskedPan The masked PAN of the card
       * @param cardholderName The cardholder name from the card
       * @param expiryDate The expiry date of the card in the format YYMM
       * @param token The token associated with this card
       * @param additionalData Optionally any other additional card data such as EMV data etc.
       *
       * @returns The newly created Card object
       */
    static from(maskedPan, cardholderName, expiryDate, token, additionalData) {
        const card = new Card_1();
        if (!maskedPan || maskedPan.replace(/[ X]/g, '').length > Card_1.MAX_PAN_DIGITS_ALLOWED) {
            throw new pre_conditions_1.InvalidArgumentError(`Masked PAN must not contain more than ${Card_1.MAX_PAN_DIGITS_ALLOWED} non-masked digits`);
        }
        card.maskedPan = maskedPan;
        card.cardholderName = cardholderName;
        card.expiryDate = expiryDate;
        card.cardToken = token;
        if (additionalData) {
            card.additionalData = additionalData;
        }
        return card;
    }
    /**
       * Get the expiry date formatted as per the provided pattern.
       *
       * See Java SimpleDateFormat for pattern documentation.
       *
       * @param pattern The moment.js pattern to format the expiry date as
       * @return The formatted expiry date, or null if expiry date not set or pattern is invalid
       */
    getFormattedExpiryDate(pattern) {
        if (this.expiryDate == null || pattern == null) {
            return null;
        }
        return moment(this.expiryDate, 'YYMM').format(pattern);
    }
};
Card.MAX_PAN_DIGITS_ALLOWED = 10;
__decorate([
    json2typescript_1.JsonProperty('maskedPan', String, true),
    __metadata("design:type", String)
], Card.prototype, "maskedPan", void 0);
__decorate([
    json2typescript_1.JsonProperty('cardholderName', String, true),
    __metadata("design:type", String)
], Card.prototype, "cardholderName", void 0);
__decorate([
    json2typescript_1.JsonProperty('expiryDate', String, true),
    __metadata("design:type", String)
], Card.prototype, "expiryDate", void 0);
__decorate([
    json2typescript_1.JsonProperty('cardToken', token_1.Token, true),
    __metadata("design:type", token_1.Token)
], Card.prototype, "cardToken", void 0);
__decorate([
    json2typescript_1.JsonProperty('additionalData', additional_data_1.AdditionalData),
    __metadata("design:type", additional_data_1.AdditionalData)
], Card.prototype, "additionalData", void 0);
Card = Card_1 = __decorate([
    json2typescript_1.JsonObject('Card')
], Card);
exports.Card = Card;
