import { Payment } from './payment';
import { Amounts } from './amounts';
import { Basket } from './basket';
import { Customer } from './customer';
import { Token } from './token';
import { AdditionalData } from './additional-data';
import { JsonOption } from '../util/json-option';
/**
 * Builder to construct {@link Payment} objects.
 */
export declare class PaymentBuilder {
    flowType: string;
    flowName?: string;
    amounts: Amounts;
    basket?: Basket;
    customer?: Customer;
    splitEnabled: boolean;
    cardToken?: Token;
    additionalData?: AdditionalData;
    source?: string;
    deviceId?: string;
    paymentMethod?: string;
    /**
       * Set what flow to use based on flow type and name.
       *
       * This ensures that the intended flow is used in the case of multiple flows for the provided type.
       *
       * The flow will determine what stages the payment goes through and what applications get called.
       *
       * See {@link PaymentSettings} for retrieving flow information.
       *
       * @param flowType The flow type
       * @param flowName The name of the flow to use
       * @return This builder
       */
    withPaymentFlow(flowType: string, flowName?: string): PaymentBuilder;
    /**
       * Set the amounts.
       *
       * This parameter is mandatory.
       *
       * @param amounts The amounts.
       * @return This builder
       */
    withAmounts(amounts: Amounts): PaymentBuilder;
    /**
       * Add a basket for this payment.
       *
       * Note that [[withAmounts]] must be called to reflect the value of the basket. This API and the flow processing service do not
       * take the basket data into account for any processing.
       *
       * @param basket The basket
       * @return This builder
       */
    withBasket(basket: Basket): PaymentBuilder;
    /**
       * Specify the payment method to use for transaction processing for this payment.
       *
       * Setting this has two effects - first it means FPS will filter payment applications based on supported methods, and secondarily it allows
       * any payment app that supports multiple methods to proceed without a user choice.
       *
       * Note that this value has no effect on flow services in general - they may still use any method of their choice to offer services such as loyalty.
       *
       * @param paymentMethod The payment method to use for this payment
       * @return This builder
       */
    withPaymentMethod(paymentMethod: string): PaymentBuilder;
    /**
       * Add customer details for this payment.
       *
       * Note that this should not be set for a split payment.
       *
       * @param customer Customer details
       * @return This builder
       */
    withCustomer(customer: Customer): PaymentBuilder;
    /**
       * Set split enabled for this payment which means it *may* be broken up into multiple sub-payments.
       *
       * It is up to the flow processing service configuration if split is enabled or not. Use {@link PaymentClient#getPaymentSettings()} to get
       * list of flow configurations and check for defined stages
       *
       * If this is not called, it is still possible that split will be enabled during the payment flow.
       *
       * @param enabled True to enable split, false to disable
       * @return This builder
       */
    withSplitEnabled(enabled: boolean): PaymentBuilder;
    /**
       * Set any previously generated card token to use for this transaction.
       *
       * @param cardToken The card {@link Token}
       * @return This builder
       */
    withCardToken(cardToken: Token): PaymentBuilder;
    /**
       * Convenience wrapper for adding additional data.
       *
       * See {@link AdditionalData#addData(String, Object[])} for more info.
       *
       * @param key    The key to use for this data
       * @param values An array of values for this data
       * @param <T>    The type of object this data is an array of
       * @return This builder
       */
    addAdditionalData(key: string, values: any | JsonOption): PaymentBuilder;
    /**
       * Set the additional data to pass via the payment.
       *
       * @param additionalData The additional data
       * @return This builder
       */
    withAdditionalData(additionalData: AdditionalData): PaymentBuilder;
    /**
       * Get the current additional data model.
       *
       * @return The additional data
       */
    getCurrentAdditionalData(): AdditionalData;
    /**
       * Set the id of the device this payment should be processed on.
       *
       * @param deviceId The id of the device to process this payment on
       * @return This builder
       */
    withDeviceId(deviceId: string): PaymentBuilder;
    /**
       *
       * @param source Set the source of this message. Usually this is the Id of POS application that is initiating this request
       * @returns This builder
       */
    withSource(source: string): PaymentBuilder;
    /**
       * Build an instance of {@link Payment} using the provided parameters.
       *
       * @return An instance of {@link Payment}
       * @throws IllegalArgumentException for invalid data or combinations of data
       */
    build(): Payment;
}
